import { DetailedHTMLProps, HTMLAttributes, ReactNode, JSX } from 'react';

type SpeechSynthesisUtteranceKeys = SpeechSynthesisUtteranceKey[];
type HighlightMode = "word" | "sentence" | "line" | "paragraph";
type SpanProps = DetailedHTMLProps<HTMLAttributes<HTMLSpanElement>, HTMLSpanElement>;
type SpeakingWord = {
    index: string;
    length: number;
} | null;
type SpeechStatus = "started" | "paused" | "stopped" | "queued";
type SpeechSynthesisErrorHandler = (error: Error) => any;
type SpeechSynthesisEventHandler = (event: SpeechSynthesisEvent) => any;
type SpeechSynthesisEventName = "word" | "sentence";
type UseSpeechOptions = {
    text: ReactNode;
    pitch?: number;
    rate?: number;
    volume?: number;
    lang?: string;
    voiceURI?: string | string[];
    autoPlay?: boolean;
    preserveUtteranceQueue?: boolean;
    highlightText?: boolean;
    showOnlyHighlightedText?: boolean;
    highlightMode?: HighlightMode;
    highlightProps?: SpanProps;
    maxChunkSize?: number;
    onError?: SpeechSynthesisErrorHandler;
    onStart?: SpeechSynthesisEventHandler;
    onResume?: SpeechSynthesisEventHandler;
    onPause?: SpeechSynthesisEventHandler;
    onStop?: SpeechSynthesisEventHandler;
    onBoundary?: SpeechSynthesisEventHandler;
    onQueueChange?: QueueChangeEventHandler;
};
type IconProps = DetailedHTMLProps<HTMLAttributes<HTMLSpanElement>, HTMLSpanElement>;
type Button = JSX.Element | string | null;
type Children = (childrenOptions: ChildrenOptions) => ReactNode;
type ChildrenOptions = {
    speechStatus?: SpeechStatus;
    isInQueue?: boolean;
    start?: Function;
    pause?: Function;
    stop?: Function;
};
type DivProps = DetailedHTMLProps<HTMLAttributes<HTMLDivElement>, HTMLDivElement>;
type SpeechProps = UseSpeechOptions & {
    id?: string;
    startBtn?: Button;
    pauseBtn?: Button;
    stopBtn?: Button;
    useStopOverPause?: boolean;
    props?: DivProps;
    children?: Children;
};
type QueueChangeEventHandler = (queue: SpeechUtterancesQueue) => any;
type SpeechQueueItem = {
    utterance: SpeechSynthesisUtterance;
    displayUtterance: SpeechSynthesisUtterance;
    setSpeechStatus: SpeechStatusUpdater;
};
type SpeechQueue = SpeechQueueItem[];
type SpeechStatusUpdater = (newStatus: SpeechStatus) => void;
type SpeechUtterancesQueue = SpeechSynthesisUtterance[];
type Index = string | number;
type State = {
    stopReason: "auto" | "manual";
};
type SpeechSynthesisUtteranceKey = keyof SpeechSynthesisUtterance;
type Words = string | Words[];

export type { Button, Children, ChildrenOptions, DivProps, HighlightMode, IconProps, Index, QueueChangeEventHandler, SpanProps, SpeakingWord, SpeechProps, SpeechQueue, SpeechQueueItem, SpeechStatus, SpeechStatusUpdater, SpeechSynthesisErrorHandler, SpeechSynthesisEventHandler, SpeechSynthesisEventName, SpeechSynthesisUtteranceKey, SpeechSynthesisUtteranceKeys, SpeechUtterancesQueue, State, UseSpeechOptions, Words };
